﻿using CMS.CustomTables;
using CMS.DataEngine;
using CMS.Helpers;
using CMS.MediaLibrary;
using CMS.SiteProvider;
using CMS.UIControls;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Text.RegularExpressions;

namespace Custom
{
    [UIElement("KenticoCloudExportModule", "KenticoCloudExportModule")]
    public partial class KenticoCloudExportModule : CMSPage
    {
        protected void Page_Load(object sender, EventArgs e)
        {
        }

        protected async void btn_Click(object sender, EventArgs e)
        {
            StringBuilder sb = new StringBuilder();
            try
            {
                List<Element> lstExoplanets = new List<Element>();

                // Prepares the code name (class name) of the custom table
                string customTableClassName = "custom.Exoplanets";

                // Gets the custom table
                DataClassInfo customTable = DataClassInfoProvider.GetDataClassInfo(customTableClassName);
                if (customTable != null)
                {
                    // Gets all data records from the custom table
                    var customTableData = CustomTableItemProvider.GetItems(customTableClassName);

                    // Loops through individual custom table records
                    foreach (CustomTableItem item in customTableData)
                    {
                        //TODO: Finish planet image integration                        
                        //string pattern = @"[0-9a-fA-F]{8}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{12}";
                        //string input = ValidationHelper.GetString(item.GetValue("PlanetImage"), "");

                        //var guid = Regex.Match(input, pattern, RegexOptions.IgnoreCase).Value;

                        // Add a new Element object to the list
                        lstExoplanets.Add(new Element
                        {
                            planetname = ValidationHelper.GetString(item.GetValue("PlanetName"), ""),
                            starname = ValidationHelper.GetString(item.GetValue("StarName"), ""),
                            yeardiscovered = ValidationHelper.GetInteger(item.GetValue("YearDiscovered"), 1900)
                        });
                    }
                }

                if (lstExoplanets.Count > 0)
                {
                    sb.Append("<h2>Exported data</h2>");
                    // Create a client to connect to Kentico Cloud 
                    var client = new HttpClient
                    {
                        BaseAddress = new Uri("https://manage.kenticocloud.com/projects/" + txtProjectID.Text + "/")
                    };
                    client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
                    client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", txtAPIKey.Text);

                    foreach (Element element in lstExoplanets)
                    {
                        Exoplanet planet = new Exoplanet();
                        planet.Name = element.planetname;
                        planet.Type = new Type { codename = "exoplanet" };
                        planet.sitemap_locations = new List<string>();
                        planet.elements = element;

                        // Create the item within the Kentico Cloud project
                        HttpResponseMessage response = await client.PostAsJsonAsync("items", planet);
                        response.EnsureSuccessStatusCode();

                        // Parse the response
                        JObject jobj = JObject.Parse(response.Content.ReadAsStringAsync().Result);


                        //TODO: Add functionality to upload planet image
                        //// Get the file info
                        //MediaFileInfo fileinfo = MediaFileInfoProvider.GetMediaFileInfo(Guid.Parse(element.planetimage), SiteContext.CurrentSiteName);

                        //if (fileinfo != null)
                        //{
                        //    // Upload the planet image
                        //    var client2 = new HttpClient
                        //    {
                        //        BaseAddress = new Uri("https://manage.kenticocloud.com/projects/" + txtProjectID.Text + "/")
                        //    };
                        //    client2.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", txtAPIKey.Text);
                        //    client2.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue(fileinfo.FileMimeType));
                        //}

                        // Update the new item with the planet details
                        HttpResponseMessage response2 = await client.PutAsJsonAsync("items/" + (string)jobj["id"].Value<string>() + "/variants/codename/default", planet);
                        response2.EnsureSuccessStatusCode();

                        sb.Append(element.planetname + " exported!<br />");
                    }

                    lblResults.Text = sb.ToString();
                }
                else
                {
                    lblResults.Text = "No planets exported!";
                }

            }
            catch (Exception ex)
            {
                sb.Append(ex.Message);
                if (ex.InnerException != null)
                {
                    sb.Append("<br />" + ex.InnerException.Message);
                }
                ShowMessage(CMS.Base.Web.UI.MessageTypeEnum.Error, "There was an error processing the request.", sb.ToString(), null, true);
            }
        }
    }

#region Classes
    public class Exoplanet
    {
        public string Name { get; set; }
        public Type Type { get; set; }
        public List<string> sitemap_locations { get; set; }
        public Element elements { get; set; }
    }

    public class Type
    {
        public string codename { get; set; }
    }

    public class Element
    {
        public string planetname { get; set; }
        public string starname { get; set; }
        public int yeardiscovered { get; set; }
        [JsonIgnore]
        public string planetimage { get; set; }
    }
#endregion
}